#!/bin/bash -le
# purpose: Create package and place it in /dock
# environment variables:
#   DIST: What mount to write the package to (default: '/dist')
#   LMATOOLS_REF: git reference (branch name) to use for lmatools (default: minvaluegrids)
#   GLMTOOLS_REF: git reference (branch name) to use for glmtools (default: ugf-newgrid)
#   GLMTOOLS_REPOS: git repository URL to use for glmtools (default: https://github.com/deeplycloudy/glmtools.git)
#   GGLM_REF: git reference (branch name) to use for this package's bash scripts (default: master)
#   GGLM_REPOS: git repository URL to use for this package's bash scripts
#       If a "/work/cspp-geo-gridded-glm" directory is already mounted
#       then that will be used and GGLM_REF will be ignored.
#   MINIFY_TARBALL: perform extra "risky" operations to make the tarball as
#       small as possible. 0 = False; True otherwise (default: True)
# note: The shebang at the top of this file is needed as-is. The '-l' will
#       load the .bashrc which allows us to use conda. The '-e' triggers exit
#       on error behavior. We cannot use '/usr/bin/env bash -le' since most
#       linux versions only parse the first two arguments of a shebang.

if [ $# -eq 1 ]; then
    version=$1
else
    echo "Usage: package.sh vX.Y.Z"
    exit 1
fi

pkg_name=cspp-geo-gridded-glm-${version}
DIST=${DIST:-"/dock"}
LMATOOLS_REF=${LMATOOLS_REF:-"minvaluegrids"}
GLMTOOLS_REF=${GLMTOOLS_REF:-"master"}
GLMTOOLS_REPOS=${GLMTOOLS_REPOS:-"https://github.com/deeplycloudy/glmtools.git"}
GGLM_REPOS=${GGLM_REPOS:-"https://gitlab.ssec.wisc.edu/cspp_geo/cspp-geo-gridded-glm.git"}
GGLM_REF=${GGLM_REF:-"master"}
MINIFY_TARBALL=${MINIFY_TARBALL:-1}

make_dockerfile() {
    cat >$DIST/Dockerfile <<EOF
# docker build . -f Dockerfile -t gitlab.ssec.wisc.edu:5555/cspp_geo/cspp-geo-gridded-glm/run_package:${version}
# docker run -it -v /path/to/data:/work --rm gitlab.ssec.wisc.edu:5555/cspp_geo/cspp-geo-gridded-glm/run_package:${version} make_glm_grids.sh OR_GLM-L2-LCFA_*.nc
FROM centos:7
RUN mkdir -p /opt/ssec && mkdir -p /work
ADD ${pkg_name}.tar.xz /opt/ssec
WORKDIR /work
ENV CSPP_GEO_GGLM_HOME=/opt/ssec/${pkg_name}
ENV PATH=\$PATH:/opt/ssec/${pkg_name}/bin
EOF

}

conda activate build

# Debug Info
which python
conda info -a
conda list --export
echo "Version specified: ${version}"
echo "lmatools reference: ${LMATOOLS_REF}"
echo "glmtools reference: ${GLMTOOLS_REF}"

# Turn on command printing here because we don't need all of the `conda`
# internal bash commands to pollute our output
set -x

# Install glmtools and related packages
# All dependencies should have been built with the buildbucket
# Careful: Could result in missing dependencies but we want reproducibility
pip install --no-deps git+https://github.com/deeplycloudy/lmatools.git@${LMATOOLS_REF}
pip install --no-deps git+https://github.com/deeplycloudy/stormdrain.git
pip install --no-deps git+${GLMTOOLS_REPOS}@${GLMTOOLS_REF}

# get the current packages files
# if the caller mounted the repository already then use that
GGLM_DIR="/work/cspp-geo-gridded-glm/gridded_glm"
if [ ! -d "${GGLM_DIR}" ]; then
    git clone -b "${GGLM_REF}" "${GGLM_REPOS}"
fi

# Build a tarball version of the current conda environment
# TODO: Add conda cleanup commands similar to what Polar2Grid uses to save space
conda_tb=conda_lmatools-${LMATOOLS_REF}_glmtools-${GLMTOOLS_REF}.tar.gz
conda clean -ay  # remove unnecessary things from conda environment
conda pack --n-threads $(nproc) -n build -o ${conda_tb}

# Build up our package directory
mkdir -p ${pkg_name}
cd ${pkg_name}
mkdir -p bin opt/conda
# Copy package scripts/data to package directory
cp $GGLM_DIR/PACKAGE_README.md ./README.md
cp $GGLM_DIR/bin/* ./bin/
# Untar the tarball so we can put things where we want
tar -xz -C ./opt/conda -f ../${conda_tb}
# Go back to original work directory
cd ..

# Perform extra "risky" operations to make the tarball as small as possible
# Taken from https://jcrist.github.io/conda-docker-tips.html
if [ $MINIFY_TARBALL -ne 0 ]; then
    find ${pkg_name} -follow -type f -name '*.a' -delete
    find ${pkg_name} -follow -type f -name '*.pyc' -delete
    find ${pkg_name} -follow -type f -name '*.js.map' -delete
    bokeh_dir="${pkg_name}/lib/python*/site-packages/bokeh/server/static"
    if [ -d $bokeh_dir ]; then
        find $bokeh_dir -follow -type f -name '*.js' ! -name '*.min.js' -delete
    fi
fi

# Create tarball of package directory
XZ_DEFAULTS="--threads=$(nproc)" tar -Jc --owner 0 --group 0 -f ${pkg_name}.tar.xz ${pkg_name}
mv ${pkg_name}.tar.xz ${DIST}/
make_dockerfile
